# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Oct-22 16:06 (EDT)
# Function: Notification Method base class
#
# $Id: NotMe.pm,v 1.5 2002/12/12 04:19:44 jaw Exp $

package NotMe;
use POSIX qw(_exit);

# but I seek my master, and my master seeks not me
#   -- Shakespeare, Two Gentlemen of Verona

%methods;

sub transmit {
    my $note = shift;
    my $meth = shift;
    my $addr = shift;
    my $msg  = shift;
    my $extra= shift;

    # Though this be madness, yet there is method in 't.
    #   -- Shakespeare, Hamlet
    if( $methods{$meth} ){

	if( ref($methods{$meth}) eq 'CODE' ){
	    $methods{$meth}->($note, $meth, $addr, $msg, $extra);
	}else{
	    if( ! fork ){
		$m = $methods{$meth};
		my $cmd  = expand( $note, $m->{command}, $meth, $addr, $msg, $extra);
		my $send = expand( $note, $m->{send},    $meth, $addr, $msg, $extra );

		# RSN - set various env vars?
		open( Q, "| $cmd" );
		print Q "$send\n";
		close Q;
		_exit(0);
	    }else{
		# parent - do nothing
	    }
	}
    }else{
	$note->loggit( $addr, "unknown method ($meth) - cannot send", 1 );
    }
}

# null notification method
sub null {}
# for debugging
sub nlog {
    my $note = shift;
    my $meth = shift;
    my $addr = shift;
    my $msg  = shift;
    my $extra= shift;

    ::loggit( "(log) NOTIFY: $msg $extra" );
}

sub expand {
    my $note = shift;
    my $txt  = shift;
    my $meth = shift;
    my $addr = shift;
    my $msg  = shift;
    my $extr = shift;

    $txt =~ s/%R/$addr/g;
    $txt =~ s/%M/$msg/g;
    $txt =~ s/%E/$extr/g;
    $txt =~ s/%N/$meth/g;
    $txt =~ s/%P/$note->{priority}/g;
    $txt =~ s/%F/$note->{mailfrom}/g;
    $txt =~ s/%S/$note->{objstate}/g;
    $txt =~ s/%I/$note->{idno}/g;
    
    my $o = $note->{obj}->unique();
    $txt =~ s/%O/$o/g;
    
    # $txt =~ ...
    
    $txt;
}

sub readconfig {
    my $cf = shift;
    my $me = {};
    
    my $line = $cf->nextline();
    my($name) = $line =~ /^Method\s+\"(.*)\"/i;
    $me->{name} = $name;

    while( defined($_ = $cf->nextline()) ){
	if( /^\s*\}/ ){
	    # done
	    last;
	}
	elsif( /:/ ){
	    my($k, $v) = split /:\s*/, $_, 2;
	    $cf->warning( "redefinition of parameter '$k'" )
		if defined $me->{$k};
	    $me->{$k} = $v;
	}
	else{
	    return $cf->error( "invalid entry in config file: '$_'" );
	}
    }

    return $cf->error( "invalid Notification Method - command not specified" )
	unless $me->{command};

    $cf->warning( "redefinition of Notification Method '$name'" )
	if $methods{$name};
    
    $methods{$name} = $me;
}



$methods{null} = \&null;
$methods{log}  = \&nlog;

$methods{qpage} = {
    command => "$::path_qpage %R",
    send    => "%M%E",
} if $::path_qpage;

$methods{mail} = {
    command => "$::path_sendmail -t",
    send    => "To: %R\nFrom: %F\nSubject: Argus%E\n\n%M\n",
} if $::path_sendmail;


1;
