# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Nov-06 18:03 (EST)
# Function: monitor my own resource usage - for testing
#
# $Id: Usage.pm,v 1.3 2002/12/09 05:10:06 jaw Exp $

# it turns out that this file is fairly useless
# NetBSD returns 0 for all of the memory usage fields

package Usage;
use Devel::Size ('total_size');

@ISA = qw(Service);
$doc = {
    package => __PACKAGE__,
    file    => __FILE__,
    isa     => [qw(Service MonEl BaseIO)],
    methods => {},
    versn  => '3.2',
    fields => {
      usage::param => {
	  attrs => ['config'],
      },
    },
};

eval { require "syscalls.ph"; $HAVE_SYSCALL_PH = 1; };
$SYS_getrusage ||= 117;	# XXX

sub probe {
    my $name = shift;

    # return undef unless $HAVE_SYSCALL_PH;	# XXX
    return ( $name =~ /^Usage/ ) ? [ 4, \&config ] : undef;
}

sub config {
    my $me = shift;
    my $cf = shift;
    
    bless $me;
    $me->init_from_config( $doc, 'usage' );
    
    if( $me->{name} =~ /Usage\/(.*)/ ){
	$me->{usage}{param} ||= $1;
    }
    
    $me->{label_right_maybe} ||= $me->{usage}{param};
    $me->{uname} = "Usage_$me->{usage}{param}";
}

sub start {
    my $me = shift;
    my( $p, $i, $ru, @r );

    $me->Service::start();
    $p = $me->{usage}{param};
    
    if( $p eq 'load' ){
	# netbsd
	if( open(F, "/kern/loadavg") ){
	    $ru = <F>;
	    my(@a) = split /\s+/, $ru;
	    $ru = $a[0] / $a[3];
	    close F;
	}else{
	    $ru = `uptime`;
	    $ru =~ s/.*:\s*([\d\.]+).*\n?/$1/;
	}
    }elsif( $p eq 'Objects' ){
	$ru = total_size( \@MonEl::all );
    }elsif( $p eq 'Notifs' ){
	$ru = keys %Notify::byid;
    }elsif( $p eq 'Sched' ){
	$ru = @BaseIO::bytime;
	
    }elsif( $p !~ /\d/ ){
    	my $c = "ps -p $$ -o $p | tail -1";
	$me->debug( "running: $c" );
	$ru = `$c`;
    }else{
	# XXX - NetBSD getrusage is broke...
	$ru = ' ' x 128;
	$i = syscall( $SYS_getrusage, 0, $ru );
	if( $i ){
	    my $m = "getrusage failed: $!";
	    ::sysproblem( "USAGE $m" );
	    return $me->isdown( $m );
	}
	@r = unpack('l*', $ru);
	if( defined($p) ){
	    $ru = $r[$p];
	}
    }
    $me->generic_test($ru, 'USAGE');
}

sub about_more {
    my $me = shift;
    my $ctl = shift;
    my( $k, $v );

    $me->Service::about_more($ctl);
    foreach $k (sort keys %{$me->{usage}}){
	$v = $me->{usage}{$k};
	next unless defined $v;
	if( ref($v) ){
	    $v = "#<REF>";
	}else{
	    $v = ::xxx_encode( $v );
	}
	$ctl->write( "usage::$k:\t$v\n" );
    }
}

    
################################################################
# global config
################################################################
Doc::register( $doc );
push @Service::probes, \&probe;

1;
