#!__PERL__
# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Nov-01 11:54 (EST)
# Function: lay paint on canvas. make pretty graphs.
#
# $Id: picasso.pl,v 1.7 2002/12/09 05:10:08 jaw Exp $

# Computers are useless. They can only give you answers.
#   -- Picasso

use lib('__LIBDIR__');
require "conf.pl";
use Image;
use GraphData;
use Getopt::Std;
getopts('bBc:C:gGlr:s:S:t:Tw:x:y:');

$which = $opt_w;
$size  = $opt_s;

$img = Image->new( $size );
$img->transparent() if $opt_T;
$img->border()      if $opt_B;
$img->logscale()    if $opt_l;  # NYI
$img->usebinary(  ) if $opt_b;
$img->usecolors($opt_c) if $opt_c;
$img->usegrid()     if $opt_g;

if( @ARGV > 1 ){
    $mode2 = 'line';
}else{
    $mode2 = $which eq 'samples' ? 'filled' : 'line';
}
$mode1 = $which eq 'samples' ? '' : $opt_S;
$mode1 = '' if $mode1 eq 'none';

# We first survey the plot, ...
foreach my $n (@ARGV){
    my $m = GraphData->new($n);
    push @us, $m;
    if( $which eq 'samples' ){
	$m->readsamples();
	$img->analyze_samples( $m->{samples} );
    }else{
	$m->readsummary( $which, 576 );	# max 576 points (~ 3 weeks)
	$img->analyze_summary( $m->{samples} );
    }
}

$img->setrange( $opt_r ) if $opt_r;
$img->clabels($opt_C, scalar(@ARGV)) if $opt_C;
$img->xlabel( $opt_x ) if $opt_x;	# draw our labels and titles 1st
$img->ylabel( $opt_y ) if $opt_y;	# we'll move the axii if needed
$img->title(  $opt_t ) if $opt_t;
$img->makeroom()       if $mode1;
$img->ytics();				# this will also move the axii
$img->xtics();
$img->axii();
$img->drawgrid() unless $opt_G;

#..., then draw the model;
#   -- Shakespeare, King Henry IV
if( $mode1 ){
    foreach my $m (@us){
	$img->draw( $mode1, $m->{samples} );
    }
}
foreach my $m (@us){
    $img->draw( $mode2, $m->{samples} );
}
    
$img->drawgrid() if $opt_G;

print $img->png();
exit 0;

sub error {
    my $msg = shift;

    print STDERR "picasso: $msg\n";
    print "Status: 500 picasso error\n";
    print "Content-Type: text/html\n\n";
    print <<EOW;
<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">
<HTML><HEAD><TITLE>ERROR</TITLE>
</HEAD><BODY BGCOLOR=\"#FFFFFF\">
<TABLE BORDER=1 BGCOLOR="#FF8888" WIDTH="95%"><TR><TD>
<TABLE><TR><TD>
<H2>ERROR&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</H2>
</TD><TD>
$msg
</TD></TR></TABLE>
</TD></TR></TABLE>
</BODY></HTML>
EOW
    exit -1;
}

